% ESAC DC Stability Shortcourse, September 13-14, 2001
% S.D. Sudhoff
% Example 4a

% Step 1: Create the s-vector.
freqmin=1.0;                        % minimum frequency considered
freqmax=1000.0;                     % maximum frequency considered
nfreq=60;                           % number of frequencies considered
s = svec1(freqmin,freqmax,nfreq);   % create the s vector for the Nyquist contour

% Step 2: Parameters of the generalized sets.
NS=8;                               % number of sides to generalized source impedance
NI=1;                               % number of intermediate point to generalized source impedance

% Step 3: Define the stability criteria to use
gm=3;                               % gain margin in dB
pm=30;                              % phase margin in degrees
tr=60;                              % truncation radious in dB (how far out into lefthand plane we go
n1=20;                              % number of points in horizontel portion of criteria (upper half)
n2=20;                              % number of points to use in sloped portion of criteria (upper half)

% Step 4: Generalized description of source
vpllmin=560.0*0.95;                 % minimum source voltage (l-l,rms)
vpllmax=560.0*1.05;                 % maximum source voltage (l-l,rms)
nv=3;                               % number of voltages to consider
wemin=2*pi*60*0.95;                 % minimum source frequency (rad/s)
wemax=2*pi*60*1.05;                 % maximum source frequency (rad/s)
nwe=3;                              % number of source frequencies to try
llpmin=4.16e-3;                     % minimum transformer leakage inductance (primary side)
llpmax=2.08e-3;                     % maximum transformer leakage inductance (seconary side)
nl=3;                               % number of reactances to try
nps=1.513;                          % turns ratio
ldc=3.5e-3;                         % dc link inductance
rdc=0.33;                           % dc link inductor resistance
cdc=1.5e-3;                         % output capacitance
rcdc=0.0;
kp=0.0;                             % controller proportional gain
ki=0.0;                             % controller integral gain
[PCM4_S] = rectsrce(vpllmin,vpllmax,nv,wemin,wemax,nwe,llpmin,llpmax,nl,nps,ldc,rdc,cdc,rcdc,kp,ki,s,NS,NI);

% Step 5: Calculated a load admitance constraint
[L_const] = esacdspec(gm,pm,tr,n1,n2,PCM4_S,s);

% Step 6: Characterize load 1 
c=25.0e-6;                          % input capacitance
r=0.01;                             % input capacitance series resistance
nv=5;                               % number of voltages to use
pmin=0.0;                           % minimum power
pmax=45000;                         % maximum power
np=10;                              % number of powers to use
% calculate the maximum dc voltage
vmax=(3.0*sqrt(2.0)/pi)*vpllmax/nps 
% calculate the minimum dc voltage
vmin0=(3.0*sqrt(2.0)/pi)*vpllmin/nps;         % no load dc voltage with minimum input
reffmax=(3/pi)*wemax*(llpmax/nps^2)+rdc;      % maximum impedance case
vmin=(vmin0+sqrt(vmin0^2-4.0*reffmax*pmax))*0.5
[L1] = cpld(c,r,vmin,vmax,nv,pmin,pmax,np,s,NS,NI);

% Step 7: See if system 1 will be stable
spaceplt(1,4,s,L_const,0.0);
hold on;
spaceplt(1,4,s,L1,0.0);
hold off;
title('Generalized Load Admittance Constraint and Load 1 Admittance');

% Step 8: Characterize load 2 and see if system 2 is stable
nnsc=1.0;                           % order of nonlinear stabilizing control
taunsc=1.0e-2;                      % time constant of nonlinear stabilizing control
[L2] = cpldnsc(c,r,nnsc,taunsc,vmin,vmax,nv,pmin,pmax,np,s,NS,NI);

% Step 9: See if system 2 will be stable
spaceplt(2,4,s,L_const,0.0);
hold on;
spaceplt(2,4,s,L2,0.0);
hold off;
title('Generalized Load Admittance Constraint and Load 2 Admittance');