% ESAC DC Stability Shortcourse, September 13-14, 2001
% S.D. Sudhoff
% Example 5

% Step 1: Stability Criteria
gm=3;                               % gain margin in dB
pm=30.0;                            % phase margin in deg
tr=40.0;                            % truncation radious in dB
n1=15.0;                            % number of points used in curve
n2=5.0;                             
[sc] = esacsc(gm,pm,tr,n1,n2);

% Step 2: Create the s-vector.
freqmin=0.01;                       % minimum frequency considered
freqmax=10000.0;                    % maximum frequency considered
nfreq=120;                          % number of frequencies considered
s = svec1(freqmin,freqmax,nfreq);   % create the s vector for the Nyquist contour

% Step 3: Decide on number of sides.
NS=8;                               % number of sides to generalized source impedance
NI=1;                               % number of intermediate point to generalized source impedance

% Step 4: Characterize Q-PCM4 (Power Supply)
vpllmin=560.0*0.95;                 % minimum source voltage (l-l,rms)
vpllmax=560.0*1.05;                 % maximum source voltage (l-l,rms)
nv=3;                               % number of voltages to consider
wemin=2*pi*60*0.95;                 % minimum source frequency (rad/s)
wemax=2*pi*60*1.05;                 % maximum source frequency (rad/s)
nwe=3;                              % number of source frequencies to try
llpmin=4.16e-3;                     % minimum transformer leakage inductance (primary side)
llpmax=2.08e-3;                     % maximum transformer leakage inductance (seconary side)
nl=3;                               % number of reactances to try
nps=1.513;                          % turns ratio
ldc=3.5e-3;                         % dc link inductance
rdc=0.33;                           % dc link inductor resistance
cdc=1.5e-3;                         % output capacitance
rcdc=0.0;                           % output capacitance ESR
kp=0.0;                             % controller proportional gain
ki=0.0;                             % controller integral gain
[PCM4_S] = rectsrce(vpllmin,vpllmax,nv,wemin,wemax,nwe,llpmin,llpmax,nl,nps,ldc,rdc,cdc,rcdc,kp,ki,s,NS,NI);

% Step 5: Characterize Q-PCM1 (SSCM) 
pmn=0.0;                            % minimum output power (W)
pmx=5.0e3;                          % maximum output power (W)
np=10;                              % number of output powers considered
vinmn=450;                          % minimum input voltage considered (V)
vinmx=525;                          % maximum input voltage considered (V)
nvin=5;                             % number of input voltages considered
voutstar=420.0;                     % commanded output voltage (V)
rin=1.0e-10;                        % input inductor resistance (Ohms)
lin=1.0e-10;                        % input inductor inductance (H)
cin=500.0e-6;                       % input capacitance (F)
rout=0.1;                           % output inductor resistance (Ohms)
cout=500.0e-6;                      % effective wye-connected input capacitance (F)
kpv=0.6;                            % proportional gain (A/V)
kiv=100.0;                          % integral gain (A/(Vs))
d=1.0;                              % droop (V/A)
nnsc=0.0;                           % nonlinear stabilizing control order
taum=0.002;                         % NSC modulator time constant
taui=0.020;                         % NSC integrator time constant
[PCM1_H11,PCM1_H12,PCM1_H21,PCM1_H22] = ...
convmod(pmn,pmx,np,vinmn,vinmx,nvin, ...
        voutstar,rin,lin,cin,rout,cout, ...
        kpv,kiv,d,taum,nnsc,taui,s,NS,NI);
                         
% Step 6: Characterize: Q-PCM2 (SSIM)
c=590e-6;                           % input capacitance
r=0.127;                            % input capacitance series resistance
vmax=420.0;                         % maximum input voltage
vmin=400.0;                         % minimum input voltage
nv=5;                               % number of voltages to use
pmin=0.0;                           % minimum power
pmax=5000;                          % maximum power
np=10;                              % number of powers to use
[PCM2_L] = cpld(c,r,vmin,vmax,nv,pmin,pmax,np,s,NS,NI);         

% Step 6: Check Stability and Analyze Q-PCM1 / Q PCM2 Interface
% Step 6a (optional): Manual Stability Check
[PCM2_L_const] = esacdspec(gm,pm,tr,n1,n2,PCM1_H12,s);
spaceplt(1,4,s,PCM2_L_const,0.0);
hold on;
spaceplt(1,4,s,PCM2_L_const,-360.0);
spaceplt(1,4,s,PCM2_L,0.0);
hold off;
title('Stability of Q-PCM1 / Q-PCM2 Interface');

% Step 6b (required): Automatic stability Check; Find Effective Load
[stable1,PCM1_PCM2_L] = HL_to_L(s,sc,PCM1_H11,PCM1_H12,PCM1_H21,PCM1_H22,PCM2_L,NS,NI);
if stable1
   disp('Q-PCM1 / Q-PCM2 Interface is stable');
else
   disp('Q-PCM1 / Q-PCM2 Interface does not satisfy stability criteria');
end

% Step 7: Parallel the Three Sets of Q-PCM 1 / Q-PCM 2 Converters
T_PCM1_PCM2_L=parL_to_L(NS,NI,PCM1_PCM2_L,PCM1_PCM2_L,PCM1_PCM2_L);
% Step 7: Now Check System Stability
% Step 7a (optional): Manual Stability Check
[T_PCM1_PCM2_L_const] = esacdspec(gm,pm,tr,n1,n2,PCM4_S,s);
spaceplt(2,4,s,T_PCM1_PCM2_L_const,0.0);
hold on;
spaceplt(2,4,s,T_PCM1_PCM2_L_const,-360.0);
spaceplt(2,4,s,T_PCM1_PCM2_L,0.0);
hold off;
title('Stability of Q-PCM4 / Remainder of System Interface');
% Step 7b: Automatic stability inspection
[sok stable2] = sinspect(s,sc,PCM4_S,T_PCM1_PCM2_L,NS,NI);
if stable2
   disp('Q-PCM4 / System Interface is stable');
else
   disp('Q-PCM4 / System Interface does not satisfy stability criteria');
end