% ESAC DC Stability Shortcourse, September 13-14, 2001
% S.D. Sudhoff
% Example 7

% Step 1: Set Up Stability Criteria
gm=3;                               % gain margin
pm=30.0;                            % phase margin
tr=40.0;                            % truncation radious
n1=15.0;                            % number of points used in horizontel trace
n2=5.0;                             % number of points used in slanted trace
[sc] = esacsc(gm,pm,tr,n1,n2);

% Step 2: Create the s-vector.
freqmin=0.01;                       % minimum frequency considered
freqmax=10000.0;                    % maximum frequency considered
nfreq=120;                          % number of frequencies considered
s = svec1(freqmin,freqmax,nfreq);   % create the s vector for the Nyquist contour

% Step 3: Decide on number of sides and interpolating points.
NS=8;                               % number of sides to generalized source impedance
NI=1;                               % number of intermediate point to generalized source impedance

% Step 4: Characterize Q-PCM4 (Power Supply)
vpllmin=560.0*0.95;                 % minimum source voltage (l-l,rms)
vpllmax=560.0*1.05;                 % maximum source voltage (l-l,rms)
nv=3;                               % number of voltages to consider
wemin=2*pi*60*0.95;                 % minimum source frequency (rad/s)
wemax=2*pi*60*1.05;                 % maximum source frequency (rad/s)
nwe=3;                              % number of source frequencies to try
llpmin=4.16e-3;                     % minimum transformer leakage inductance (primary side)
llpmax=2.08e-3;                     % maximum transformer leakage inductance (seconary side)
nl=3;                               % number of reactances to try
nps=1.513;                          % turns ratio
ldc=3.5e-3;                         % dc link inductance
rdc=0.33;                           % dc link inductor resistance
cdc=1.5e-3;                         % output capacitance
rcdc=0.0;                           % output capacitor ESR
kp=0.0;                             % controller proportional gain
ki=0.0;                             % controller integral gain
[PCM4_S] = rectsrce(vpllmin,vpllmax,nv,wemin,wemax,nwe,llpmin,llpmax,nl,nps,ldc,rdc,cdc,rcdc,kp,ki,s,NS,NI);

% Step 5: Characterize Q-PCM1 (SSCM) 
pmn=0.0;                            % minimum output power (W)
pmx=5.0e3;                          % maximum output power (W)
np=10;                              % number of output powers considered
vinmn=450;                          % minimum input voltage considered (V)
vinmx=525;                          % maximum input voltage considered (V)
nvin=5;                             % number of input voltages considered
voutstar=420.0;                     % commanded output voltage (V)
rin=1.0e-10;                        % input inductor resistance (Ohms)
lin=1.0e-10;                        % input inductor inductance (H)
cin=500.0e-6;                       % input capacitance (F)
rout=0.1;                           % output inductor resistance (Ohms)
cout=500.0e-6;                      % effective wye-connected input capacitance (F)
kpv=0.6;                            % proportional gain (A/V)
kiv=100.0;                          % integral gain (A/(Vs))
d=1.0;                              % droop (V/A)
nnsc=0.0;                           % nonlinear stabilizing control order
taum=0.002;                         % NSC modulator time constant
taui=0.020;                         % NSC integrator time constant
[PCM1_H11,PCM1_H12,PCM1_H21,PCM1_H22] = convmod(pmn,pmx,np,vinmn,vinmx,nvin, ...
                                        voutstar,rin,lin,cin,rout,cout, ...
                                        kpv,kiv,d,taum,nnsc,taui,s,NS,NI);
                         
% Step 6: Characterize: Q-PCM2 (SSIM)
c=590e-6;                           % input capacitance
r=0.127;                            % input capacitance series resistance
vmax=420.0;                         % maximum input voltage
vmin=400.0;                         % minimum input voltage
nv=5;                               % number of voltages to use
pmin=0.0;                           % minimum power
pmax=5000;                          % maximum power
np=10;                              % number of powers to use
[PCM2_L] = cpld(c,r,vmin,vmax,nv,pmin,pmax,np,s,NS,NI);         

% Step 7: Analyze a Single Zone 
% Step 7a: Single Zone Interace
[ok,SZ_Y11,SZ_Y12,SZ_Y21,SZ_Y22,PCM1_H12inv,PCM1_H12_par_PCM1_H12] =  ...
HLHp_to_Y(s,sc,PCM1_H11,PCM1_H12,PCM1_H21,PCM1_H22, ...
          PCM1_H11,PCM1_H12,PCM1_H21,PCM1_H22,PCM2_L,NS,NI);
if ok 
   disp('Dual Q-PCM1 to Q-PCM2 interface is stable');
else
   disp('Dual Q-PCM1 to Q-PCM2 interface is unstable');
end
% Step 7b: Check HLHp Stability Manually
[PCM1_H12inv_const] = esacdspec(gm,pm,tr,n1,n2,PCM1_H12,s);
spaceplt(1,4,s,PCM1_H12inv_const,0.0);
hold on;
spaceplt(1,4,s,PCM1_H12inv_const,-360.0);
spaceplt(1,4,s,PCM1_H12inv,0.0);
hold off;
title('Stability of Q-PCM1 to Q-PCM1 Interface');
[PCM2_L_const]=esacdspec(gm,pm,tr,n1,n2,PCM1_H12_par_PCM1_H12,s);
spaceplt(2,4,s,PCM2_L_const,0.0);
hold on;
spaceplt(2,4,s,PCM2_L_const,-360.0);
spaceplt(2,4,s,PCM2_L,0.0);
hold off;           
title('Stability of Dual Q-PCM1 to Q-PCM2 Interface');

% Step 8 Parallel the Three Zonal Loads
[TZ_Y11,TZ_Y12,TZ_Y21,TZ_Y22] = ...
parY_to_Y(NS,NI,SZ_Y11,SZ_Y12,SZ_Y21,SZ_Y22, ...
          SZ_Y11,SZ_Y12,SZ_Y21,SZ_Y22, ...
          SZ_Y11,SZ_Y12,SZ_Y21,SZ_Y22);

% Step 9   Combine loads with starboard source
% Step 9a: Compute the effective load of the zonal loads and starborad power supply 
[stable,TZplusPCM4_L] = YS_to_L(s,sc,TZ_Y11,TZ_Y12,TZ_Y21,TZ_Y22,PCM4_S,NS,NI);
if stable
   disp('Zonal Load / Starboard Power Supply Interface is stable');
else
   disp('Zonal Load / Starboard Power Supply Interface does not satisfy stability criteria');
end
% Step 9b: Manual check to see if zonal loads and 2nd source form a stable subsyste
[L_const_of_PCM4] = esacdspec(gm,pm,tr,n1,n2,PCM4_S,s);
spaceplt(3,4,s,L_const_of_PCM4,0.0);
hold on;
spaceplt(3,4,s,L_const_of_PCM4,-360.0);
spaceplt(3,4,s,TZ_Y22,0.0);
hold off;
title('Stability of Three Zonal Loads and Port Power Supply Interface');
spaceplt(4,6,s,L_const_of_PCM4,-20.0);
hold on;
spaceplt(4,6,s,TZ_Y22,-20.0);
title('Stability of Three Zonal Loads and Port Power Supply Interface');
hold off;

% Step 10 Check System Stability at the Port Power Supply Interface
% Stet 10a: Check it Automatically
[sok ok] = sinspect(s,sc,PCM4_S,TZplusPCM4_L,NS,NI);
if ok
   disp('Port PCM4 Rest of System Interface is Stable');
else
   disp('Port PCM4 Rest of System Interface does not Satisfy Stability Criteria');
end
% Step 10b: Check it Manually
spaceplt(5,4,s,L_const_of_PCM4,0.0);
hold on;
spaceplt(5,4,s,L_const_of_PCM4,-360.0);
spaceplt(5,4,s,TZplusPCM4_L,0.0);
title('Stability of Port PCM4 Interface');
hold off;
spaceplt(6,6,s,L_const_of_PCM4,-20.0);
hold on;
spaceplt(6,6,s,TZplusPCM4_L,-20.0);
title('Stability of Port PCM4 Interface');
hold off;